@@ -140,6 +140,12 @@ struct SyncOptions {
      */
     Callback_t m_startCallback;
 
+    /**
+     * called while configuration is prepared for sync, see
+     * SyncContext::prepare()
+     */
+    Callback_t m_prepareCallback;
+
     boost::shared_ptr<TransportAgent> m_transport;
 
     SyncOptions(SyncMode syncMode = SYNC_NONE,
@@ -174,6 +180,7 @@ struct SyncOptions {
     SyncOptions &setRetryDuration(int retryDuration) { m_retryDuration = retryDuration; return *this; }
     SyncOptions &setRetryInterval(int retryInterval) { m_retryInterval = retryInterval; return *this; }
     SyncOptions &setStartCallback(const Callback_t &callback) { m_startCallback = callback; return *this; }
+    SyncOptions &setPrepareCallback(const Callback_t &callback) { m_prepareCallback = callback; return *this; }
     SyncOptions &setTransportAgent(const boost::shared_ptr<TransportAgent> transport)
                                   {m_transport = transport; return *this;}
 
@@ -225,6 +232,9 @@ class ClientTest {
     ClientTest(int serverSleepSec = 0, const std::string &serverLog= "");
     virtual ~ClientTest();
 
+    /** set up before running a test */
+    virtual void setup() { }
+
     /** cleanup function to be called when shutting down testing */
     typedef void (*Cleanup_t)(void);
 
@@ -290,7 +300,9 @@ class ClientTest {
     /**
      * utility function for importing items with blank lines as separator
      */
-    static int import(ClientTest &client, TestingSyncSource &source, const char *file, std::string &realfile);
+    static int import(ClientTest &client, TestingSyncSource &source,
+                      const ClientTestConfig &config,
+                      const char *file, std::string &realfile);
 
     /**
      * utility function for comparing vCard and iCal files with the external
@@ -298,6 +310,12 @@ class ClientTest {
      */
     static bool compare(ClientTest &client, const char *fileA, const char *fileB);
 
+    /**
+     * utility function: update a vCard or iCalendar item by inserting "MOD-" into
+     * FN, N, resp. SUMMARY; used for ClientTestConfig::update
+     */
+    static void update(std::string &item);
+
     struct ClientTestConfig config;
 
     /**
@@ -308,11 +326,10 @@ class ClientTest {
      * Some of the test cases are compiled into the library, other
      * depend on the auxiliary files from the "test" directory.
      * Currently supported types:
-     * - vcard30 = vCard 3.0 contacts
-     * - vcard21 = vCard 2.1 contacts
-     * - ical20 = iCal 2.0 events
-     * - vcal10 = vCal 1.0 events
-     * - itodo20 = iCal 2.0 tasks
+     * - eds_contact = vCard 3.0 contacts, with Evolution extensions
+     * - eds_event = iCalendar 2.0 events, as used by Evolution
+     * - eds_task = iCalendar 2.0 tasks, as used by Evolution
+     * - eds_memo = iCalendar 2.0 journals, as used by Evolution
      */
     static void getTestData(const char *type, Config &config);
 
@@ -460,6 +477,9 @@ public:
         createSourceB(co.createSourceB, cl, sourceParam, false)
         {}
 
+    /** set up before running a test */
+    virtual void setUp() { client.setup(); }
+
     /**
      * adds the supported tests to the instance itself;
      * this is the function that a derived class can override
@@ -472,9 +492,10 @@ public:
      * regardless whether the data source already contains items or not
      *
      * @param relaxed   if true, then disable some of the additional checks after adding the item
+     * @retval inserted    actual data that was inserted, optional
      * @return the LUID of the inserted item
      */
-    virtual std::string insert(CreateSource createSource, const char *data, bool relaxed = false);
+    virtual std::string insert(CreateSource createSource, const char *data, bool relaxed = false, std::string *inserted = NULL);
 
     /**
      * assumes that exactly one element is currently inserted and updates it with the given item
@@ -504,6 +525,11 @@ public:
      */
     virtual bool compareDatabases(const char *refFile, TestingSyncSource &copy, bool raiseAssert = true);
 
+    /**
+     * compare data in source with vararg list of std::string pointers, NULL terminated
+     */
+    void compareDatabases(TestingSyncSource &copy, ...);
+
     /**
      * insert artificial items, number of them determined by TEST_EVOLUTION_NUM_ITEMS
      * unless passed explicitly
@@ -515,6 +541,12 @@ public:
      * @return number of items inserted
      */
     virtual std::list<std::string> insertManyItems(CreateSource createSource, int startIndex = 1, int numItems = 0, int size = -1);
+    virtual std::list<std::string> insertManyItems(TestingSyncSource *source, int startIndex = 1, int numItems = 0, int size = -1);
+
+    /**
+     * update every single item, using config.update
+     */
+    virtual void updateData(CreateSource createSource);
 
     /**
      * create an artificial item for the current database
@@ -576,8 +608,16 @@ public:
     SyncTests(const std::string &name, ClientTest &cl, std::vector<int> sourceIndices, bool isClientA = true);
     ~SyncTests();
 
-    /** adds the supported tests to the instance itself */
-    virtual void addTests();
+    /**
+     * adds the supported tests to the instance itself
+     * @param isFirstSource     the tests are getting generated for a single source,
+     *                          the one which was listed first; some tests are the
+     *                          same for all sources and should only be run once
+     */
+    virtual void addTests(bool isFirstSource = false);
+
+    /** set up before running a test */
+    virtual void setUp() { client.setup(); }
 
 protected:
     /** list with all local test classes for manipulating the sources and their index in the client */
@@ -694,6 +734,7 @@ protected:
     virtual void testConversion();
     virtual void testItems();
     virtual void testItemsXML();
+    virtual void testExtensions();
     virtual void testAddUpdate();
 
     // test copying with maxMsg and no large object support
@@ -764,6 +805,16 @@ protected:
     virtual void testResendServerUpdate();
     virtual void testResendFull();
 
+    virtual void testResendProxyClientAdd();
+    virtual void testResendProxyClientRemove();
+    virtual void testResendProxyClientUpdate();
+    virtual void testResendProxyServerAdd();
+    virtual void testResendProxyServerRemove();
+    virtual void testResendProxyServerUpdate();
+    virtual void testResendProxyFull();
+
+    virtual void testTimeout();
+
     /**
      * implements testMaxMsg(), testLargeObject(), testLargeObjectEncoded()
      * using a sequence of items with varying sizes
@@ -781,6 +832,7 @@ protected:
         SyncPrefix prefix(logPrefix, *this);
         doSync(options);
     }
+    virtual void postSync(int res, const std::string &logname);
 };
 
 /*
@@ -807,6 +859,13 @@ public:
     ~TransportWrapper() {
     }
 
+    /**
+     * -1 for wrappers which are meant to be used without message resending,
+     * otherwise the number x for which "interrupt" <= x will lead to
+     * an aborted sync (0 for TransportResendInjector, 2 for TransportResendProxy)
+     */
+    virtual int getResendFailureThreshold() { return -1; }
+
     virtual int getMessageCount() { return m_messageCount; }
 
     virtual void setURL(const std::string &url) { m_wrappedAgent->setURL(url); }
@@ -825,8 +884,7 @@ public:
         m_wrappedAgent.reset();
     }
     virtual Status wait(bool noReply = false) { return m_status; }
-    virtual void setCallback (TransportCallback cb, void *udata, int interval) 
-    { return m_wrappedAgent->setCallback(cb, udata, interval);}
+    virtual void setTimeout(int seconds) { m_wrappedAgent->setTimeout(seconds); }
 };
 
 /** assert equality, include string in message if unequal */
